package esclient

import (
	"context"
	"encoding/json"
	"fmt"
	"git.168cad.top/go/logger/log"
	"github.com/olivere/elastic/v7"
	"sort"
	"time"
)

func getCurrentIndex() string {
	if Conf.LogIndexNamePrefix == "" {
		return Conf.IndexNamePrefix + time.Now().Format("20060102")
	} else {
		return Conf.LogIndexNamePrefix + time.Now().Format("20060102")
	}
}

type ServerLogModel struct {
	ServerName   string `json:"serverName"`
	Env          string `json:"env"`
	TimeStamp    int64  `json:"timeStamp"`
	ThreadNo     string `json:"threadNo"`
	ServerIp     string `json:"serverIp"`
	ClientIp     string `json:"clientIp"`
	Token        string `json:"token"`
	CustomerId   uint32 `json:"customerId"`
	ClientSource string `json:"clientSource"`
	Url          string `json:"url"`
	Method       string `json:"method"`
	Request      string `json:"request"`
	Response     string `json:"response"`
	DealTimes    int64  `json:"dealTimes"`
}

func InsertEsLog(logInfo interface{}) {
	indexName := getCurrentIndex()
	put, err := getRedisClient().Index().
		Index(indexName).
		BodyJson(logInfo).
		Do(context.Background())
	if err != nil {
		log.Error(fmt.Sprintf("insert es log fail：%s", err.Error()))
	} else {
		log.Debug(fmt.Sprintf("insert es log success：%s", put.Id))
	}
}

func GroupField(indexName, field string, size, minCount int, excludeValues []string) ([]byte,error) {
	if size == 0 {
		size = 30
	}
	aggregationQuery := elastic.NewTermsAggregation().Field(field).MinDocCount(minCount).Size(size)
	for _, excludeValue := range excludeValues {
		aggregationQuery.ExcludeValues(excludeValue)
	}
	if log.IsDebug() {
		source, _ := aggregationQuery.Source()
		queryByte, _ := json.Marshal(source)
		log.Info(fmt.Sprintf("查询条件：%s", string(queryByte)))
	}
	res, err := getRedisClient().Search(indexName).
		Aggregation("group_field", aggregationQuery).
		From(0).
		Size(0).
		Do(context.Background())
	if err != nil {
		return nil,err
	}
	return json.Marshal(res.Aggregations)
}

func QueryLogs(indexName, filed, value string, from, size int) (interface{},error) {
	if from <= 0 {
		from = 0
	}
	if size <= 0 {
		size = 10
	}
	query := elastic.NewBoolQuery().Must(elastic.NewTermQuery(filed, value))
	return getRedisClient().Search(indexName).Query(query).
		From(from).
		Size(size).
		Do(context.Background())
}

type IndexInfo struct {
	Name  string `json:"name"`
	Count int    `json:"count"`
	Size  string `json:"size"`
}

func AllIndex() (interface{},error) {
	res, err := getRedisClient().CatIndices().Do(context.Background())
	if err != nil {
		return nil,err
	}
	var keys []string
	var sizeMap = make(map[string]IndexInfo)
	for _, row := range res {
		keys = append(keys, row.Index)
		sizeMap[row.Index] = IndexInfo{row.Index, row.DocsCount, row.StoreSize}
	}
	sort.Strings(keys)
	data := make([]IndexInfo, len(keys))
	for i, key := range keys {
		data[i] = sizeMap[key]
	}
	return data, nil
}

func DeleteIndex(indexName string) (interface{},error) {
	return getRedisClient().DeleteIndex(indexName).Do(context.Background())
}
